@@ -1,12 +1,25 @@
 import datetime
 import logging
+import sys
 from typing import Optional
 
 import structlog
 
 from . import config, encoding, git, package, run, source
+from .log import SECRET_MASKER
+from .source.local import Local
 
 structlog.configure(
+    processors=[
+        structlog.processors.add_log_level,
+        SECRET_MASKER.process_event,
+        structlog.processors.StackInfoRenderer(),
+        structlog.dev.set_exc_info,
+        structlog.processors.TimeStamper(fmt="%Y-%m-%d %H:%M.%S", utc=False),
+        structlog.dev.ConsoleRenderer(
+            colors=sys.stdout is not None and sys.stdout.isatty()
+        ),
+    ],
     wrapper_class=structlog.make_filtering_bound_logger(logging.INFO),
 )
 
@@ -65,7 +78,7 @@ class RepoRun:
             return
 
         work_dir = self.git.prepare(repo)
-        tpl_mapping = {"repo_name": repo.name, "repo_project": repo.project}
+        tpl_mapping = create_template_mapping(repo)
         pr = source.PullRequest(
             matcher.name,
             self.opts.config.pr_title_prefix,
@@ -74,26 +87,8 @@ class RepoRun:
             matcher.pr_body,
             matcher.pr_title,
         )
+        apply_actions(pkgs, repo, matcher, tpl_mapping, work_dir)
         has_changes = False
-        for a in matcher.actions:
-            log.debug(
-                "Applying action from run",
-                action=a.__class__.__name__,
-                run=matcher.name,
-                repo=str(repo),
-            )
-            a.apply(work_dir, tpl_mapping)
-
-        for pkg in pkgs:
-            for a in pkg.actions:
-                log.debug(
-                    "Applying action from package",
-                    action=a.__class__.__name__,
-                    pkg=pkg.name,
-                    repo=str(repo),
-                )
-                a.apply(work_dir, tpl_mapping)
-
         if self.git.has_changes(work_dir) is True:
             log.debug("Committing changes", repo=str(repo))
             self.git.commit_changes(work_dir, matcher.commit_msg)
@@ -145,6 +140,33 @@ class RepoRun:
                 repo.merge_pull_request(pr_identifier)
 
 
+def apply_actions(
+    pkgs: list[package.Package],
+    repo: source.Repository,
+    rrun: run.Run,
+    tpl_mapping: dict,
+    work_dir: str,
+) -> None:
+    for a in rrun.actions:
+        log.debug(
+            "Applying action from run",
+            action=a.__class__.__name__,
+            run=rrun.name,
+            repo=str(repo),
+        )
+        a.apply(work_dir, tpl_mapping)
+
+    for pkg in pkgs:
+        for a in pkg.actions:
+            log.debug(
+                "Applying action from package",
+                action=a.__class__.__name__,
+                pkg=pkg.name,
+                repo=str(repo),
+            )
+            a.apply(work_dir, tpl_mapping)
+
+
 def execute(opts: Options) -> bool:
     log_level = logging.getLevelName(opts.config.log_level.upper())
     structlog.configure(
@@ -162,6 +184,7 @@ def execute(opts: Options) -> bool:
     log.info("Repositories returned by sources", count=len(repositories))
     gitc = git.Git(
         matcher.branch(opts.config.git.branch_prefix),
+        opts.config.git.clone_options,
         opts.config.git.data_dir,
         opts.config.git.user_name,
         opts.config.git.user_email,
@@ -170,6 +193,7 @@ def execute(opts: Options) -> bool:
     success = True
     for repo in repositories:
         if matcher.match(repo) is False:
+            log.debug("Repository does not match", repository=str(repo))
             continue
 
         log.info("Matched repository", repository=str(repo))
@@ -185,6 +209,23 @@ def execute(opts: Options) -> bool:
     return success
 
 
+def execute_local(
+    directory: str, repo_source: str, repo_project: str, repo_name: str, opts: Options
+) -> None:
+    log_level = logging.getLevelName(opts.config.log_level.upper())
+    structlog.configure(
+        wrapper_class=structlog.make_filtering_bound_logger(log_level),
+    )
+
+    pkg_reader = package.PackageReader(opts.encoding_registry)
+    pkgs = pkg_reader.read_packages(opts.packages_paths)
+    matcher = run.read(opts.matcher_path)
+    pkgs_to_apply = find_packages(matcher.packages, pkgs)
+    repo = Local(repo_source, repo_project, repo_name)
+    tpl_mapping: dict[str, str] = create_template_mapping(repo)
+    apply_actions(pkgs_to_apply, repo, matcher, tpl_mapping, directory)
+
+
 def options_from_config(path: str) -> Options:
     cfg = config.read_config_from_file(path)
     return config_to_options(cfg)
@@ -228,3 +269,11 @@ def find_packages(
         raise RuntimeError(f"unknown packages {unknown_pkg_names}")
 
     return pkgs_found
+
+
+def create_template_mapping(repo: source.Repository) -> dict[str, str]:
+    return {
+        "repo_name": repo.name,
+        "repo_project": repo.project,
+        "repo_source": repo.source,
+    }
