@@ -1,21 +1,27 @@
 import datetime
-from typing import Any, Union
+import fnmatch
+import io
+import os.path
+from typing import Any, TextIO, Union
 from urllib.parse import urlparse
 
 import gitlab
 import structlog
+from gitlab import GitlabGetError
 from gitlab.base import RESTObjectList
 from gitlab.v4.objects import Project as GitlabProject
 from gitlab.v4.objects.merge_requests import ProjectMergeRequest as GitlabMergeRequest
 
-from .source import Base, PullRequest, Repository
+from ..log import SECRET_MASKER
+from .source import Base, PullRequest, Repository, add_credentials_to_url
 
-log = structlog.get_logger(source="gitlab")
+log: structlog.stdlib.BoundLogger = structlog.get_logger(source="gitlab")
 
 
 class GitlabRepository(Repository):
-    def __init__(self, project: GitlabProject, url: str):
+    def __init__(self, project: GitlabProject, token: str, url: str):
         self._project = project
+        self.token = token
         self.url = url
 
     @property
@@ -24,7 +30,10 @@ class GitlabRepository(Repository):
 
     @property
     def clone_url(self) -> str:
-        return self._project.http_url_to_repo
+        # Value of username does not matter to GitLab.
+        return add_credentials_to_url(
+            url=self._project.http_url_to_repo, password=self.token, username="rcmt"
+        )
 
     def create_pull_request(self, branch: str, pr: PullRequest) -> None:
         log.debug(
@@ -55,37 +64,55 @@ class GitlabRepository(Repository):
             f"GitLab API returned an unexpected object {mrs.__class__.__name__}"
         )
 
-    def has_file(self, path: str) -> bool:
+    def get_file(self, path: str) -> TextIO:
         try:
-            self._project.files.get(file_path=path, ref=self.base_branch)
-        except gitlab.exceptions.GitlabGetError as e:
-            if e.response_code == 404:
-                return False
-            else:
-                raise e
+            content = self._project.files.get(
+                file_path=path, ref=self.base_branch
+            ).decode()
+            if content is None:
+                log.warning("Decoded content is None", repo=str(self), file=path)
+                raise FileNotFoundError("decoded content is None")
 
-        return True
+            return io.StringIO(content.decode("utf-8"))
+        except GitlabGetError:
+            raise FileNotFoundError("file does not exist in repository")
+
+    def has_file(self, path: str) -> bool:
+        directory = os.path.dirname(path)
+        file = os.path.basename(path)
+        tree = self._project.repository_tree(path=directory)
+        for entry in tree:
+            if entry["type"] != "blob":
+                continue
+
+            if fnmatch.fnmatch(entry["path"], file):
+                return True
+
+        return False
 
     def has_successful_pr_build(self, identifier: GitlabMergeRequest) -> bool:
-        pipelines = identifier.pipelines.list()
-        if len(pipelines) == 0:
-            log.debug("No pipelines", repo=str(self), id=identifier.get_id())
+        failed = False
+        for commit_status in self._project.commits.get(identifier.sha).statuses.list():
+            if commit_status.allow_failure is True:
+                continue
+
+            if commit_status.status != "success":
+                log.debug(
+                    "MR check failed",
+                    repo=str(self),
+                    id=identifier.get_id(),
+                    name=commit_status.name,
+                    status=commit_status.status,
+                )
+                failed = True
+
+        if failed is False:
+            log.debug(
+                "All MR checks successful", repo=str(self), id=identifier.get_id()
+            )
             return True
 
-        for pl in pipelines:
-            if pl.sha == identifier.sha and pl.status != "success":
-                log.warn(
-                    "Pipeline not successful",
-                    pipeline_id=pl.id,
-                    repo=str(self),
-                    status=pl.status,
-                )
-                return False
-
-        log.debug(
-            "All pipeline runs successful", repo=str(self), id=identifier.get_id()
-        )
-        return True
+        return False
 
     def is_pr_closed(self, mr: GitlabMergeRequest) -> bool:
         return mr.state == "closed" or mr.state == "locked"
@@ -110,7 +137,7 @@ class GitlabRepository(Repository):
 
     @property
     def project(self) -> str:
-        return self._project.namespace["path"]
+        return self._project.namespace["full_path"]
 
     @property
     def source(self) -> str:
@@ -122,6 +149,8 @@ class Gitlab(Base):
         self.client = gitlab.Gitlab(url, private_token=private_token)
         self.url = urlparse(url).netloc
 
+        SECRET_MASKER.add_secret(private_token)
+
     def list_repositories(self) -> list[Repository]:
         log.debug("start fetching repositories")
         projects = self.client.projects.list(
@@ -129,7 +158,7 @@ class Gitlab(Base):
         )
         repositories: list[Repository] = []
         for p in projects:
-            repositories.append(GitlabRepository(project=p, url=self.url))  # type: ignore
+            repositories.append(GitlabRepository(project=p, token=self.client.private_token, url=self.url))  # type: ignore
 
         log.debug("finished fetching repositories")
         return repositories
